import re
import sys
from collections import OrderedDict
from functools import partial, wraps
from operator import itemgetter
from string import punctuation
from time import sleep

from .trakt_utils.parsers import APIParser
from .trakt_utils.type_hints import (
    IntOrStr,
    Literal,
    LiteralCategory,
    LiteralSection,
    Optional,
    StrTuple,
)
from .trakt_utils.exceptions import THException
from .trakt_utils.utils import (
    BeautifulSoup,
    enumerate_at_one,
    get_datetime,
    get_terminal_size,
    removefix,
)


class TraktHubViewer:
    """
    A class to view the contents of the Trakt.tv.
    
    #### Attributes:
        - `category`: The category to view.
        - `section`: The section to view.
        - `html_contents`: The HTML contents of the page.
    
    #### Methods:
        - `get_contents`: Get the cleaned contents.
        - `print_contents`: Print the cleaned contents.
    
    #### Raises:
        - `THException`: If any of the arguments are invalid
            or if the section is not valid for the category.
    
    #### Notes:
        - The `category` attribute must be a valid category.
        - The `section` attribute must be a valid section for the category.
    """
    
    __slots__ = (
        "_category",
        "_section",
        "_html_contents",
    )

    def __init__(
        self,
        category: LiteralCategory = "movies",
        section: LiteralSection = "",
        *,
        html_contents: BeautifulSoup = None,
    ):
        self._category = category
        self._section = section
        self._html_contents = html_contents

    def _th_viewer():
        def decorator(func):
            @wraps(func)
            def wrapper(self, *args, **kwargs):
                parser = func(self, *args, **kwargs)
                html_contents = parser.parse_html_contents(parser.contents)
                section = self._query if not args else args[0]
                return TraktHubViewer(
                    self._category, section, html_contents=html_contents
                ).get_contents()

            return wrapper

        return decorator

    def _find_all(self, tag: str, *, text: bool = False, strip: bool = False, **kwargs):
        tag_contents = self._html_contents.find_all(tag, **kwargs)
        if any((text, strip)):
            format_func = lambda x: x.get_text(strip=True) if strip else x.text
            return [format_func(i) for i in tag_contents]
        return tag_contents

    def _clean_contents(self):
        text_findall_func = partial(self._find_all, text=True)
        common_func = partial(text_findall_func, "div", class_="titles")

        def second_common_func(contents):
            pattern = re.compile(r"(\w+)(.*?)\s(\d{4})$")
            return {
                idx: {
                    "Title": match.group(1) + match.group(2),
                    "Year": int(match.group(3)),
                }
                for idx, i in enumerate_at_one(contents)
                if (match := pattern.match(i))
            }

        cleaned_contents = None

        match (self._category, self._section):
            # ^ -------------------Match Case for (Movies/Shows)----------------- ^ #
            case ("shows", "trending") | ("movies", "trending"):
                # Unclean Example: 41 people watchingSonic the Hedgehog 3 2024
                trending = common_func()
                pattern = re.compile(r"(\d+)\s+people watching(.+?)\s+(\d{4})$")
                cleaned_contents = {
                    idx: {
                        "Title": match.group(2).strip(),
                        "Watch Count": int(match.group(1)),
                        "Year": int(match.group(3)),
                    }
                    for idx, i in enumerate_at_one(trending)
                    if (match := pattern.match(i))
                }

            case ("shows", "popular") | ("movies", "popular"):
                # Unclean Example: Deadpool 2016
                popular_contents = common_func()
                cleaned_contents = second_common_func(popular_contents)
            case ("shows", "anticipated") | ("movies", "anticipated"):
                # Different html tag
                # Unclean Example: Daredevil: Born Again 2025
                anticipated = text_findall_func("a", class_="titles-link")
                cleaned_contents = second_common_func(anticipated)
            case ("movies", "boxoffice"):
                # Example: $36,000,000Dog Man 2025
                boxoffice = common_func()
                pattern = re.compile(r"(\$\d[\d,]*)(\D+)(\d{4})$")
                cleaned_contents = {
                    idx: {
                        "Title": match.group(2).strip(),
                        "Total Budget": match.group(1),
                        "Year": int(match.group(3)),
                    }
                    for idx, i in enumerate_at_one(boxoffice)
                    if (match := pattern.match(i))
                }

            # ^ -------------------------------------------------------------- ^ #

            # ^ -------------------Match Case for (Calendars)----------------- ^ #
            case ("calendars", "shows"):
                # Example: ' 1:00 amCBS7x11 Welcome to the e-Neighborhood'
                calendar_shows = self._find_all("div", class_="titles has-worded-image")
                pattern = re.compile(r"(\d{1,3}x\d{1,3})\s(\D+)")

                cleaned_contents = {
                    idx: {
                        "Title": (
                            _title_contents := pattern.match(
                                i.find(class_="titles-link").text
                            )
                        ).group(2),
                        "Network": i.find(class_="generic").text,
                        **{
                            k: _title_contents.group(1).split("x")[v]
                            for k, v in (("Season", 0), ("Episode", 1))
                        },
                        "Time": i.h4.get_text(strip=True),
                    }
                    for idx, i in enumerate_at_one(calendar_shows)
                }

                #!> FINISH CALENDARS SECTIONS
            # ^ --------------------------------------------------------- ^ #

            # ^ -------------------Match Case for (People)----------------- ^ #
            case ("people", section):
                common_func = partial(self._find_all, "div")
                person_credits = {
                    idx: i.find(class_="ellipsify").text
                    for idx, i in enumerate_at_one(common_func(class_="titles"))
                }

                person_descr_details = common_func(class_="col-lg-8 col-md-7")
                person_description = next(
                    (i.text.lstrip(i.ul.text) for i in person_descr_details)
                )

                # Example: ['Age60', 'GenderMale', 'Birthday1964-09-02', 'BirthplaceBeirut, Lebanon', 'Known ForActing']
                person_stats = ("Age", "Gender", "Birthday", "Birthplace", "Known For")
                person_details_uncleaned = next(
                    (
                        [j.text for j in i.ul.find_all("li")]
                        for i in person_descr_details
                    )
                )

                cleaned_contents = OrderedDict(
                    {
                        "Person": self._section.lstrip("/").title(),
                        **{
                            j: i.removeprefix(j)
                            for i, j in zip(person_details_uncleaned, person_stats)
                        },
                        "Description": person_description,
                        "Credits": person_credits,
                    }
                )
            # ^ --------------------------------------------------------- ^#

        # ^ No default case needed as all cases are covered and will raise an exception if not found ^ #

        return cleaned_contents

    def get_contents(self):
        return self._clean_contents()

    @staticmethod
    def print_contents(contents):
        if isinstance(contents, (bool, str)):
            # For cases for all:
            #   ~ '--<arg>' commands (Returned obj is type str)
            #   ~ Is-Functions (Returned obj is type boolean)
            print(contents)
            sys.exit()

        cli_args = sys.argv
        verbose = False

        if "--verbose" in cli_args:
            cli_args.remove("--verbose")
            verbose = True

        function_name = query = cli_args[1]
        terminal_col, _terminal_lines = get_terminal_size()

        def print_header(*args):
            current_dt = get_datetime(with_time=True)
            sep = "-" * (terminal_col // 2)
            header = "TraktHub - {} {}"
            print(sep, end="\n\n")
            header = header.format(
                *args,
            ).center(((terminal_col + len(header)) // 2) - len("TraktHub"), "-")
            print(header, end="\n\n")
            print(sep)
            print(f"Time Now: {current_dt}", end="\n\n")

        def verbose_output(code: int, c=""):
            match code:
                case 1:
                    print(
                        "~ Trakt.tv has been successfully accessed and the data has been retrieved."
                    )
                    print("~ Formatting the data for display...")
                    sleep(0.5)
                case 2:
                    print(f"~ Creating a formatted display for '{c}'.")
                    print("~ This may take a few seconds...")
                    sleep(2)
                    print("~ The formatted data will now be displayed.", end="\n\n")
                    sleep(1)

        value_string = partial("{:>5}• {k}{:>5}: {v}".format, " ", " ")
        
        if verbose:
            verbose_output(1)

        if function_name.startswith("get"):
            # All Get-Functions (Returned obj is type dict)

            # 'get-trending' -> 'trending'
            f_name = function_name.split("-")[-1]
            if function_name.endswith("boxoffice"):
                f_name = "Current"
                cat = "boxoffice"
            else:
                # 'movies' or 'shows'
                cat = cli_args[2][3:]

            if verbose:
                verbose_output(2)

            print_header(*(i.title() for i in (f_name, cat)))

            # popular and anticipated only have title and year
            common_keys = ("Title", "Year")
            diff_set = lambda x: len(set(x) - set(common_keys))
            for idx, v in contents.items():
                match diff_set(v):
                    case 0:
                        # Get-Popular/Anticipated functions
                        print(f"{idx}: {v['Title']} ({v['Year']})")
                    case 1:
                        # Boxoffice
                        def _format(*args):
                            print(
                                f"{idx}-{v['Title']} ({v['Year']})",
                                "\n{:>5}• {}: {}".format(" ", *args),
                            )

                        uncommon_keys = ("Total Budget", "Watch Count")

                        if cat == "boxoffice":
                            _format(uncommon_keys[0], v[uncommon_keys[0]])
                        elif cat in ("movies", "shows"):
                            _format(uncommon_keys[1], v[uncommon_keys[1]])
        elif query in ("-q", "--query"):
            cat = cli_args[-1]
            
            if verbose:
                verbose_output(2, cat)
            
            if cat == "movies":
                header_title = (
                    contents["Basic Info"][i] for i in ("Title", "Release Year")
                )
                print_header(*header_title)

                for header_section, header_values in contents.items():
                    print(f"\n\n[{header_section}]")
                    for key, value in header_values.items():
                        if isinstance(value, (list, tuple)):
                            if header_section == "Cast":
                                p = re.compile(r"^(.*?)\s*\[((.*?))\]$")
                                actors = tuple(
                                    (m.group(1), f"as {m.group(2)}")
                                    for av in value
                                    if (m := p.match(av))
                                )
                                for actors_name, role_name in actors:
                                    print(value_string(k=actors_name, v=role_name))
                            else:
                                value = ", ".join(map(str, value))
                                print(value_string(k=key, v=value))
                        else:
                            print(value_string(k=key, v=value))
            elif cat == "people":
                person = contents["Person"]
                keyval_sep = "-" * 6

                print_header(person, "")

                for key, value in contents.items():
                    if key == "Credits":
                        print(f"• {key}:")
                        for idx, credit in value.items():
                            print(f"{'':2}{idx}:{keyval_sep} {credit}")
                    else:
                        if key == "Description":
                            value = "\n" + value + "\n"
                            keyval_sep = ""
                        print(f"• {key}:{keyval_sep} {value}")
            elif cat == "shows":
                show_title = contents["Basic Info"]["Title"]
                print_header(show_title, "")
                
                for header_section, header_values in contents.items():
                    print(f"[{header_section}]")
                    for section, values in header_values.items():
                        if isinstance(values, tuple):
                            values = ", ".join(values)
                        print(value_string(k=section, v=values))


class TraktHub:
    """
    A class for interacting with the `Trakt.tv` database by categorizing and retrieving
    information on movies, TV shows, people, and calendar events.

    ### Attributes:
    `MAIN_DB` : str
        The base URL for Trakt.tv (`https://trakt.tv/`), used as the primary database.

    `CATEGORIES` : Tuple[str, ...]
        A tuple representing the main content categories available on Trakt:
        - people           : Queries related to actors, directors, and crew members.
        - shows            : Queries related to TV shows.
        - movies           : Queries related to movies.
        - calendars        : Queries related to release calendars.

    `SHOW_SECTIONS` : Tuple[str, ...]
        Sections specific to TV shows:
        - trending         : Lists currently trending TV shows.
        - popular          : Lists popular TV shows based on user ratings and activity.
        - anticipated      : Lists most anticipated TV shows.

    `MOVIE_SECTIONS` : Tuple[str, ...]
        Sections specific to movies, including those from `_SHOW_SECTIONS`:
        - trending         : Lists currently trending movies.
        - popular          : Lists popular movies based on user ratings and activity.
        - anticipated      : Lists most anticipated movies.
        - boxoffice        : Lists movies currently performing well at the box office.

    `CALENDARS_SECTIONS` : Tuple[str, ...]
        Sections specific to Trakt's calendar feature, which organizes upcoming releases:
        - people           : Calendar section related to people (e.g., actor appearances).
        - shows            : Calendar section related to TV shows.
        - movies           : Calendar section related to movies.
        - premieres        : Lists upcoming movie/show premieres.
        - new-shows        : Lists newly released TV shows.
        - finales          : Lists upcoming season/series finales.
        - dvd              : Lists upcoming DVD/Blu-ray releases.

    `ALL_SECTIONS` : Tuple[str, ...]
        A combined tuple of all available sections across shows, movies, and calendars.

    ### Parameters:
        search : str, optional
            The search term used to query movies, TV shows, or people. Default is an empty string.

        category : Literal["people", "movie", "show", "calendar"], optional
            The category of content to search within. Must be one of:
            - "people"   : Searches for actors, directors, or crew members.
            - "movie"    : Searches for movies.
            - "show"     : Searches for TV shows.
            - "calendar" : Searches within the calendar feature.

        section : Optional[Literal["trending", "popular", "anticipated", "boxoffice", "premieres", "new-shows", "finales", "dvd"]], optional
            A subsection within the selected category to refine search results. Default is an empty string.

        year : Optional[IntOrStr], optional
            An optional argument to specify a year filter for movies or TV shows.
            If an error occurs during querying, a valid `year` argument may be required.
    
    
    #### Methods:
        - `track_hub`: Track the hub for the provided section.
        - `track_person`: Track the person for the provided query.
        - `search`: Search for the provided query.
    
    #### Raises:
        - `THException`: If any of the arguments are invalid
            or if the section is not valid for the category.
    """
    
    MAIN_DB: str = "https://trakt.tv/"
    CATEGORIES: StrTuple = ("people", "shows", "movies", "calendars")
    SHOW_SECTIONS: StrTuple = ("trending", "popular", "anticipated")
    MOVIE_SECTIONS: StrTuple = (*SHOW_SECTIONS, "boxoffice")
    CALENDARS_SECTIONS: StrTuple = (
        *CATEGORIES[:-1],
        "premieres",
        "new-shows",
        "finales",
        "dvd",
    )
    _ALL_SECTIONS: StrTuple = (*SHOW_SECTIONS, *MOVIE_SECTIONS, *CALENDARS_SECTIONS)

    _APIParser = APIParser

    __slots__ = ("_query", "_category", "_main_url", "_page_number")

    def __init__(
        self,
        *,
        query: Optional[str] = "",
        category: Literal["people", "movies", "shows", "calendars"],
        **kwargs,
    ):
        self._query, self._category, self._page_number = self._validate_args(
            query, category, **kwargs
        )
        self._main_url = self._get_url()

    def _get_url(self):
        main_url = self.MAIN_DB + self._category
        if self._query or self._category == "people":
            main_url += self._query
        return main_url

    def _validate_args(self, *args, **kwargs):
        query, category = args
        page_number = kwargs.get("page_number", "")

        if any((map(lambda x: not isinstance(x, str), (query, category)))):
            raise THException("All arguments must be strings.")

        if not isinstance(page_number, IntOrStr):
            raise THException(
                f"Page number must be an integer or a string, not {type(page_number).__name__}."
            )

        if category not in (categories := self.CATEGORIES):
            raise THException(
                f"{category!r} is invalid and must be one of the following:\n{categories}."
            )

        if query or category == "people":
            cleaned_query = query.translate(str.maketrans("", "", punctuation))
            query = "/" + "-".join(cleaned_query.split())

        return query, category, page_number

    def _validate_section(self, section: str):
        movie_sections, show_sections, calendars_sections = (
            self.MOVIE_SECTIONS,
            self.SHOW_SECTIONS,
            self.CALENDARS_SECTIONS,
        )

        if section not in self._ALL_SECTIONS:
            raise THException(
                f"{section!r} is not a valid section.\nSection Options:"
                f"\nMovie Sections: {movie_sections}"
                f"\nShow Sections: {show_sections}"
                f"\nCalendar Sections: {calendars_sections}"
            )

        cat = self._category

        def __raise_exception(main_sections):
            _all_sections = {
                "movies": movie_sections,
                "shows": show_sections,
                "calendars": calendars_sections,
            }
            correct_cat = next((k for k, v in _all_sections.items() if section in v))
            if section not in main_sections:
                raise THException(
                    f"{section!r} is not a valid section for category {cat!r}."
                    f"\nThis section is for category {correct_cat!r}."
                )

        # Check if section is in the correct category
        match cat:
            case "movies":
                __raise_exception(movie_sections)
            case "shows":
                __raise_exception(show_sections)
            case "calendars":
                __raise_exception(calendars_sections)

        if pg := self._page_number:
            section += f"?page={pg}"

        return section

    @TraktHubViewer._th_viewer()
    def track_hub(
        self,
        section: LiteralSection,
    ):
        valid_section = self._validate_section(section)
        return self._APIParser(self._main_url, valid_section)

    @TraktHubViewer._th_viewer()
    def track_person(self):
        return self._APIParser(self._main_url)

    def _search_show(self, __contents):
        show_title = self._main_url.split("/")[-1].split("-")[0].title()
        show_stats = __contents.find_all(
            "div",
            class_="col-md-10 col-md-offset-2 col-sm-9 col-sm-offset-3 ul-wrapper",
        )

        def str_translate(str_obj):
            return str_obj.translate(str.maketrans("#", " ")).strip()

        rating_stats_unclean = next(
            [
                i.get_text(strip=True, separator="#")
                for i in i.find_all("div", class_="number")
            ]
            for i in show_stats
        )
        rating_stats_unclean.pop(1)
        (
            loved_votes,
            imdb_stats,
            tmdb_stats,
            fresh_value,
            audience,
            streaming_rank,
            watchers_count,
            num_of_plays,
            collected_count,
            num_of_comments,
            num_of_lists,
            num_favorited,
        ) = rating_stats_unclean

        loved_perc, loved_votes = str_translate(removefix(loved_votes, "votes")).split()
        imdb_score, imdb_nums = imdb_stats.split("#")
        tmdb_score, tmdb_nums = tmdb_stats.split("#")
        fresh_value = str_translate(fresh_value)
        audience = str_translate(removefix(audience, "Audience"))
        justwatch_score, *justwatch_trend = str_translate(streaming_rank).split()
        justwatch_trend = " ".join(justwatch_trend)
        (
            watchers_count,
            num_of_plays,
            collected_count,
            num_of_comments,
            num_of_lists,
            num_favorited,
        ) = [
            str_translate(removefix(*i))
            for i in (
                (watchers_count, "watchers"),
                (num_of_plays, "plays"),
                (collected_count, "collected"),
                (num_of_comments, "comments"),
                (num_of_lists, "lists"),
                (num_favorited, "favorited"),
            )
        ]

        show_details = __contents.find_all("div", class_="col-lg-8 col-md-7")
        spoiler = next((i.find("div", id="tagline").text for i in show_details))
        description = next(
            (i.find("div", class_="readmore").text for i in show_details)
        )
        num_of_seasons = next(
            (
                i.find("a", class_="season-count").text
                for i in __contents.find_all(
                    "div", class_="col-md-2 col-sm-3 hidden-xs sticky-wrapper"
                )
            )
        )

        organized_data = OrderedDict(
            {
                "Basic Info": OrderedDict(
                    {
                        "Title": show_title,
                        "Total Seasons": num_of_seasons,
                    }
                ),
                "Ratings": OrderedDict(
                    {
                        "Loved %": (loved_perc, loved_votes),
                        "IMDb": (imdb_score, imdb_nums),
                        "TMDb": (tmdb_score, tmdb_nums),
                        "Rotten Tomatoe": fresh_value,
                        "JustWatch": (justwatch_score, justwatch_trend),
                        "Audience %": audience,
                    }
                ),
                "Total Engagement": OrderedDict(
                    {
                        "Watchers": watchers_count,
                        "Plays": num_of_plays,
                        "Collected": collected_count,
                        "Comments": num_of_comments,
                        "Personal Lists": num_of_lists,
                        "Favorited": num_favorited,
                    }
                ),
                "Narrative": OrderedDict(
                    {
                        "Spoiler": spoiler,
                        "Description": description,
                    }
                ),
            }
        )

        return organized_data

    def search(self):
        parser = self._APIParser(self._main_url)
        parsed_contents = parser.parse_html_contents(parser.contents)
        if self._category == "shows":
            return self._search_show(parsed_contents)

        flix_title_contents = parsed_contents.find(
            "div",
            class_="col-md-10 col-md-offset-2 col-sm-9 col-sm-offset-3 mobile-title",
        ).h1.get_text(separator="#")

        try:
            flix_title, release_year, flix_mature_rating = map(
                str.strip, flix_title_contents.split("#")
            )
        except ValueError:
            raise THException(
                "Unable to parse the contents for the provided query."
                "\nSome contents may be missing on URL the page."
            )

        loved_percentage, num_of_votes = (
            parsed_contents.find("div", class_=i).text for i in ("rating", "votes")
        )

        uncleaned_stats = next(
            (
                [
                    i.get_text(separator="#", strip=True).split("#")
                    for i in i.find_all("div", class_="number")
                ]
                for i in parsed_contents.find_all("ul", class_="stats")
            )
        )
        space_join = lambda s: " ".join(s)
        imdb_rating, imdb_num_reviews = uncleaned_stats[0]
        tmdb_rating, tmdb_num_reviews = uncleaned_stats[1]
        rotten_rating = space_join(uncleaned_stats[2])

        (
            audience_percent,
            metacritic_rating,
            num_watchers,
            num_plays,
            num_collected,
            num_comments,
            num_lists,
            num_favorited,
        ) = map(itemgetter(0), uncleaned_stats[3:])

        uncleaned_metadata = parsed_contents.find("div", class_="col-lg-8 col-md-7")
        release_date = next((i.span.text for i in uncleaned_metadata if i.span))
        _years_ago = get_datetime().year - int(release_date[:4])
        years_ago = "{ya} year{s} ago".format(
            ya=_years_ago, s="s" if _years_ago != 1 else ""
        )

        # Runtime
        runtime_in_hours = next(
            (
                _i.text
                for i in uncleaned_metadata
                if (_i := i.find("span", class_="humanized-minutes"))
            )
        )
        _time_match = re.match(r"(?:(\d+)h)?\s*(?:(\d+)m)?", runtime_in_hours)
        if _time_match:
            hours = int(_time_match.group(1) or 0)
            minutes = int(_time_match.group(2) or 0)

        runtime_in_minutes = f"{(hours * 60) + minutes}m"

        directors = next(
            (
                list(
                    set(i.meta["content"] for i in i.find_all("span", class_="hidden"))
                )
                for i in uncleaned_metadata
                if i.find("span", class_="hidden")
            )
        )
        director = directors[0]
        writers = next(
            (
                [i.meta["content"] for i in i.find_all(itemprop="writer")]
                for i in uncleaned_metadata
                if i.find("span", class_="hidden", itemprop="writer")
            )
        )

        country = next(
            (
                removefix(i.text, "Country", post=False)
                for i in uncleaned_metadata.find_all("li", itemprop="countryOfOrigin")
            )
        )

        _languages = next(
            (
                [
                    removefix(i.text, "Languages", post=False)
                    for i in i.find_all("li")
                    if "Languages" in str(i)
                ]
                for i in uncleaned_metadata
                if i.find("li", itemprop="countryOfOrigin")
            )
        )[0].split(",")
        languages = _languages[0] if len(_languages) == 1 else tuple(_languages)

        _studios = next(
            (
                [
                    removefix(i.text, "Studios", post=False)
                    for i in i.find_all("li")
                    if "Studios" in str(i)
                ]
                for i in uncleaned_metadata
                if i.find("li", itemprop="countryOfOrigin")
            )
        )
        studios = [
            i[: i.find("+") - 1].strip() if "+" in i else i.strip()
            for i in "".join(_studios).split(",")
        ]
        genres = [i.text for i in uncleaned_metadata.find_all("span", itemprop="genre")]
        spoiler, description = next(
            (
                [
                    i.find("div", id="tagline").text,
                    i.find("div", class_="readmore").text,
                ]
                for i in uncleaned_metadata.find_all(
                    "div", {"data-spoiler-movie-id": True}
                )
            )
        )
        _actors_table = [
            [i.find(class_=j).text for j in ("name", "character")]
            for i in parsed_contents.find_all("li", itemprop="actor")
        ]
        actors_table = tuple(i + f" [{j}]" for i, j in _actors_table)

        organized_data = OrderedDict(
            {
                "Basic Info": OrderedDict(
                    {
                        "Title": flix_title,
                        "Release Year": release_year,
                        "Content Rating": flix_mature_rating,
                    }
                ),
                "Ratings": OrderedDict(
                    {
                        "Loved %": (loved_percentage, num_of_votes),
                        "IMDb": (imdb_rating, imdb_num_reviews),
                        "TMDb": (tmdb_rating, tmdb_num_reviews),
                        "Rotten Tomatoe": rotten_rating,
                        "Metacritic": metacritic_rating,
                        "Audience %": audience_percent,
                    }
                ),
                "Total Engagement": OrderedDict(
                    {
                        "Watchers": num_watchers,
                        "Plays": num_plays,
                        "Collected": num_collected,
                        "Comments": num_comments,
                        "Personal Lists": num_lists,
                        "Favorited": num_favorited,
                    }
                ),
                "Release Details": OrderedDict(
                    {
                        "Release Date": (release_date, years_ago),
                        "Runtime": (runtime_in_hours, runtime_in_minutes),
                    }
                ),
                "Production": OrderedDict(
                    {
                        "Country": country,
                        "Languages": languages,
                        "Studios": studios,
                        "Genres": genres,
                        "Director": director,
                        "Directors": directors,
                        "Writers": writers,
                    }
                ),
                "Narrative": OrderedDict(
                    {
                        "Spoiler": spoiler,
                        "Description": description,
                    }
                ),
                "Cast": OrderedDict(
                    {
                        "Actors": actors_table,
                    }
                ),
            }
        )

        return organized_data


__all__ = (
    "TraktHub",
    "TraktHubViewer",
)



from argparse import ArgumentParser

from .parsers import _metadata_parser
from ..trakt_hub import TraktHubViewer
from ..trakt_functions.functions import (
    get_anticipated,
    get_boxoffice,
    get_popular,
    get_trending,
    is_anticipated,
    is_popular,
    is_trending,
    trakt_query,
)


def cli_parser():
    arg_parser = ArgumentParser(
        description="A CLI for parsing and viewing Trakt-TV data."
    )
    sub_parsers = arg_parser.add_subparsers(dest="command", help="All CLI options.")

    def _add_args(parser):
        def wrapper(*args, **kwargs):
            if all(("action" not in kwargs, all(s.startswith("--") for s in args))):
                kwargs["action"] = "store_true"
            else:
                kwargs["type"] = str
            return parser.add_argument(*args, **kwargs)

        return wrapper

    # TraktHub-Arguments
    _add_args(arg_parser)("-q", "--query")
    _add_args(arg_parser)("-c", "--category")

    # Main-Commands
    main_args = _add_args(arg_parser)
    main_args("--version", help="Display the current version of 'trakt_hub'.")
    main_args("--author", help="Display the author of 'trakt_hub'.")
    main_args("--license", help="Display the license of 'trakt_hub'.")
    main_args("--description", help="Display the description of 'trakt_hub'.")
    main_args("--url", help="Display the GitHub URL of 'trakt_hub'.")
    main_args("--verbose", help="Enable verbose output.")

    # Get-Boxoffice
    boxoffice = sub_parsers.add_parser(
        "get-boxoffice",
        argument_default="movies",
        description="Get the budget of current box office movies.",
    )

    # Get-Trending
    trending = sub_parsers.add_parser("get-trending", description="Get trending data.")
    trending_args = _add_args(trending)
    trending_args("--tshows", help="Get current trending shows.")
    trending_args("--tmovies", help="Get current trending movies.")

    # Get-Popular
    popular = sub_parsers.add_parser("get-popular", description="Get popular data.")
    popular_args = _add_args(popular)
    popular_args("--pshows", help="Get current popular shows.")
    popular_args("--pmovies", help="Get current popular movies.")

    # Get-Anticipated
    anticipated = sub_parsers.add_parser(
        "get-anticipated", description="Get anticipated data."
    )
    anticipated_args = _add_args(anticipated)
    anticipated_args("--ashows", help="Get current anticipated shows.")
    anticipated_args("--amovies", help="Get current anticipated movies.")

    # Is-Trending
    istrending = sub_parsers.add_parser(
        "is-trending", description="Check if a show or movie is trending."
    )
    istrending_args = _add_args(istrending)
    istrending_args("-q", "--query", help="Movie or show to check if it is trending.")
    istrending_args(
        "-c", "--category", help="Specify the category of the show or movie."
    )

    # Is-Anticipated
    isanticipated = sub_parsers.add_parser(
        "is-anticipated", description="Check if a show or movie is anticipated."
    )
    isanticipated_args = _add_args(isanticipated)
    isanticipated_args(
        "-q", "--query", help="Movie or show to check if it is trending."
    )
    isanticipated_args(
        "-c", "--category", help="Specify the category of the show or movie."
    )

    # Is-Popular
    ispopular = sub_parsers.add_parser(
        "is-popular", description="Check if a show or movie is popular."
    )
    ispopular_args = _add_args(ispopular)
    ispopular_args("-q", "--query", help="Movie or show to check if it is popular.")
    ispopular_args(
        "-c", "--category", help="Specify the category of the show or movie."
    )

    metadata = _metadata_parser()

    args = arg_parser.parse_args()

    def mult_args(func):
        q, c = map(
            lambda x: getattr(args, x[x[-1] in args.__dict__]),
            (["q", "query"], ["c", "category"]),
        )
        return func(query=q, category=c)

    # Main-Arguments
    if args.version:
        return metadata["version"]
    elif args.author:
        return metadata["author"]
    elif args.license:
        return metadata["license"]
    elif args.description:
        return metadata["description"]
    elif args.url:
        return metadata["url"]

    # 'get_<category>' function Arguments
    elif args.command == "get-boxoffice":
        return get_boxoffice("movies")
    elif args.command == "get-trending":
        if args.tshows:
            return get_trending("shows")
        return get_trending("movies")
    elif args.command == "get-popular":
        if args.pshows:
            return get_popular("shows")
        return get_popular("movies")
    elif args.command == "get-anticipated":
        if args.ashows:
            return get_anticipated("shows")
        return get_anticipated("movies")

    # 'is_<category>' function Arguments
    elif args.command == "is-trending":
        return mult_args(is_trending)
    elif args.command == "is-popular":
        return mult_args(is_popular)
    elif args.command == "is-anticipated":
        return mult_args(is_anticipated)

    # Main Query for TraktHub
    return mult_args(trakt_query)


if __name__ == "__main__":
    TraktHubViewer.print_contents(cli_parser())



from pathlib import Path
from typing import (
    Any,
    Callable,
    Iterable,
    Literal,
    Optional,
    Type,
    Union,
)

IntOrStr = Union[int, str]
PathLike = Union[str, Path]
StrTuple = tuple[str, ...]
MoviesOnly = Literal["movies"]
MoviesOrShows = Literal["movies", "shows"]
LiteralCategory = Literal["people", "movies", "shows", "calendars"]
LiteralSection = Literal[
    "trending",
    "popular",
    "anticipated",
    "boxoffice",
    "premieres",
    "new-shows",
    "finales",
    "dvd",
]



from functools import wraps

from ..trakt_hub import TraktHub
from ..trakt_utils.exceptions import THException
from ..trakt_utils.utils import best_match, popkwargs, page_executor


def _get_args(args=("query", "category"), **kwargs):
    kwargs.update({"default_value": ""})
    return popkwargs(*args, **kwargs)


def validate_args_wrapper(all_args: bool = False):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if not any((args, kwargs)):
                raise THException(
                    "Function {func_name!r} requires {num_args} to be provided.".format(
                        func_name=func.__name__,
                        num_args="all arguments" if all_args else "an argument",
                    )
                )
            return func(*args, **kwargs)

        return wrapper

    return decorator


def trakt_viewer_wrapper(get_func: str):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            cat = kwargs.get("category", *args)
            if get_func == "boxoffice":
                cat = "movies"
            return page_executor(TraktHub, cat, get_func)

        return wrapper

    return decorator


def is_functions_wrapper(get_func: str):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            q, cat, kwargs = _get_args(*args, **kwargs)
            executed_pages = page_executor(TraktHub, cat, get_func)
            titles = [i["Title"] for i in executed_pages.values()]
            if not (found_match := best_match(q, titles)):
                return False
            _, score, _ = found_match
            return score >= 90

        return wrapper

    return decorator


def query_viewer_wrapper(__func):
    @wraps(__func)
    def wrapper(*args, **kwargs):
        q, cat, kwargs = _get_args(*args, **kwargs)
        tk_hub = TraktHub(query=q, category=cat)
        return getattr(tk_hub, ["search", "track_person"][cat == "people"])()

    return wrapper


__all__ = tuple(k for k in globals() if k.endswith("wrapper"))



from .functions import (
    get_anticipated,
    get_boxoffice,
    get_popular,
    get_trending,
    is_anticipated,
    is_popular,
    is_trending,
    trakt_query,
)


__all__ = (
    "get_anticipated",
    "get_popular",
    "get_trending",
    "is_anticipated",
    "is_popular",
    "is_trending",
    "trakt_query",
)



import calendar
import shutil
from bs4 import BeautifulSoup
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
from datetime import datetime as dt
from functools import partial
from pathlib import Path
from urllib.parse import urlparse
from rapidfuzz import fuzz, process

from .exceptions import ExecutorException, FileException, ParserException, THException
from .type_hints import Any, Callable, Iterable, PathLike


def validate_path(file_path: PathLike, is_file: bool = False):
    try:
        fp = Path(file_path).resolve()
    except TypeError:
        raise FileException(
            f"The provided file path {file_path!r} is not a valid path and must be a string or a Path object."
        )
    if is_file and not fp.is_file():
        raise FileException(
            f"The provided file path \033[33m{fp!r}\033[0m is not a valid file path and must be a file."
            "\nOtherwise, disable the 'is_file' argument."
        )
    elif not any((fp.is_absolute(), fp.exists())):
        raise FileException(
            f"The provided file path \033[33m{fp!r}\033[0m is not a valid path and must be an existing path."
        )
    return fp


def clean_url(url: str):
    url_only = urlparse(url).netloc
    if not url_only:
        raise ParserException(f"The provided URL {url!r} is not a valid URL.")
    return url_only


def best_match(s: str, value: Iterable[str], extract_single: bool = True, **kwargs):
    pe = getattr(process, "extract" if not extract_single else "extractOne")
    return pe(s.lower(), (i.lower() for i in value), scorer=fuzz.ratio, **kwargs)


def get_terminal_size(col_or_lines: str = ""):
    ts = shutil.get_terminal_size()
    return getattr(ts, col_or_lines) if col_or_lines else ts


def soupify(contents: str, markup: str = "html.parser"):
    def _soup(*args):
        try:
            return BeautifulSoup(*args)
        except KeyError:
            raise ParserException(
                f"The provided contents {args[0]!r} is not a valid content and must be a string or a file-like object."
            )

    return _soup(contents, markup)


def popkwargs(*args, **kwargs):
    df_value = kwargs.pop("default_value", None)
    return *(kwargs.pop(k, df_value) for k in args), kwargs


def page_merger(page_contents):
    first_page = next(page_contents)
    starting_idx = len(first_page) + 1

    for page in page_contents:
        for idx, values in enumerate(page.values(), start=starting_idx):
            first_page[idx] = values

    return first_page


def page_executor(
    thub: Callable, category: str, func_type: str, merge_pages: bool = True
):
    # func_type -> Any of of the 'Get' functions
    thub_func = lambda pg: thub(category=category, page_number=pg).track_hub(func_type)
    page_contents = executor(thub_func, range(1, 6))
    if merge_pages:
        return page_merger(page_contents)
    return page_contents


def removefix(_string, obj, *, post: bool = True):
    return getattr(_string, "removesuffix" if post else "removeprefix")(obj)


def executor(func: Callable, *args, **kwargs):
    max_w, epool, kwargs = popkwargs("max_workers", "epool", **kwargs)
    maxw_p = lambda p: partial(p, max_workers=max_w)
    if all((max_w is not None, not isinstance(max_w, int))):
        raise ExecutorException(
            f"The provided {max_w = !r} is not a valid argument and must be {None!r} or a positive integer."
        )
    _exec = list(map(maxw_p, (ThreadPoolExecutor, ProcessPoolExecutor)))[
        epool == "ppex"
    ]
    yield from _exec().map(func, *args, **kwargs)


def get_datetime(increment_day: int = 0, with_time: bool = False):
    _dt = dt.now()
    date, day = _dt.date(), _dt.day
    try:
        if not with_time:
            return date.replace(day=day + increment_day)
        return _dt.strftime("%m/%d/%Y %I:%M %p")
    except ValueError:
        # For cases above 31 days
        month = calendar.month_name[date.month]
        month_range = calendar.monthrange(date.year, date.month)[1]
        raise THException(
            f"Cannot increment the day by {increment_day} as it exceeds the month of {month} which has {month_range} days."
        )


def enumerate_at_one(obj: Any):
    return enumerate(obj, start=1)



import inspect

from ..trakt_utils.type_hints import LiteralCategory, MoviesOnly, MoviesOrShows
from .wrappers import (
    is_functions_wrapper,
    query_viewer_wrapper,
    trakt_viewer_wrapper,
    validate_args_wrapper,
)


@validate_args_wrapper()
@trakt_viewer_wrapper("trending")
def get_trending(category: MoviesOrShows) -> dict:
    """
    Get the trending movies or shows on Trakt.tv.
    
    #### Args:
        - `category`: The category to get the trending data for.
            ~ "movies"
            ~ "shows"
    
    #### Returns:
        - A dictionary of the trending movies or shows.
    """
    pass


@validate_args_wrapper()
@trakt_viewer_wrapper("popular")
def get_popular(category: MoviesOrShows) -> dict:
    """
    Get the popular movies or shows on Trakt.tv.
    
    #### Args:
        - `category`: The category to get the popular data for.
            ~ "movies"
            ~ "shows"
    
    #### Returns:
        - A dictionary of the popular movies or shows.
    """
    pass


@validate_args_wrapper()
@trakt_viewer_wrapper("anticipated")
def get_anticipated(category: MoviesOrShows) -> dict:
    """
    Get the anticipated movies or shows on Trakt.tv.
    
    #### Args:
        - `category`: The category to get the anticipated data for.
            ~ "movies"
            ~ "shows"
    
    #### Returns:
        - A dictionary of the anticipated movies or shows.
    """
    pass


@trakt_viewer_wrapper("boxoffice")
def get_boxoffice(category: MoviesOnly) -> dict:
    """
    Get the current box office movies.
    
    #### Args:
        - `category`: The category to get the box office data for.
            ~ "movies"
    
    #### Returns:
        - A dictionary of the current box office movies.
    """
    pass


@validate_args_wrapper(all_args=True)
@is_functions_wrapper("trending")
def is_trending(query: str, category: LiteralCategory) -> bool:
    """
    Check if the query is trending on Trakt.tv.
    
    #### Args:
        - `query`: The query to check if it is trending.
        - `category`: The category of the query.
            ~ "movies"
            ~ "shows"
            ~ "people"
            ~ "calendars"
    
    #### Returns:
        - A boolean indicating if the query is trending.
    """
    pass


@validate_args_wrapper(all_args=True)
@is_functions_wrapper("popular")
def is_popular(query: str, category: LiteralCategory) -> bool:
    """
    Check if the query is popular on Trakt.tv.
    
    #### Args:
        - `query`: The query to check if it is popular.
        - `category`: The category of the query.
            ~ "movies"
            ~ "shows"
            ~ "people"
            ~ "calendars"
    
    #### Returns:
        - A boolean indicating if the query is popular.
    """
    pass


@validate_args_wrapper(all_args=True)
@is_functions_wrapper("anticipated")
def is_anticipated(query: str, category: LiteralCategory) -> bool:
    """
    Check if the query is anticipated on Trakt.tv.
    
    #### Args:
        - `query`: The query to check if it is anticipated.
        - `category`: The category of the query.
            ~ "movies"
            ~ "shows"
    
    #### Returns:
        - A boolean indicating if the query is anticipated.
    """
    pass


@validate_args_wrapper(all_args=True)
@query_viewer_wrapper
def trakt_query(query: str, category: LiteralCategory) -> dict:
    """
    Query Trakt.tv for the given query and category.
    
    #### Args:
        - `query`: The query to search for.
        - `category`: The category to search in.
            ~ "people"
            ~ "movies"
            ~ "shows"
            ~ "calendars"
    """
    pass


__all__ = tuple(
    k
    for k, v in globals().items()
    if all(
        (
            (k not in ("isfunction", "wraps")),
            inspect.isfunction(v),
            not any((k.startswith("_"), k.endswith("wrapper"))),
        )
    )
)



class ConnectionException(BaseException):
    def __init__(self, *args):
        super().__init__(*args)


class ExecutorException(BaseException):
    def __init__(Self, *args):
        super().__init__(*args)


class FileException(BaseException):
    def __init__(self, *args):
        super().__init__(*args)


class ConfigException(BaseException):
    def __init__(self, *args):
        super().__init__(*args)


class ParserException(BaseException):
    def __init__(self, *args):
        super().__init__(*args)


class THException(BaseException):
    def __init__(self, *args):
        super().__init__(*args)


class CLIException(BaseException):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



import asyncio
import tomllib
from aiohttp import ClientSession, TCPConnector
from aiohttp.client_exceptions import (
    ClientConnectionError,
    ClientResponseError,
    ContentTypeError,
    InvalidURL,
    ServerDisconnectedError,
)
from async_lru import alru_cache
from bs4 import BeautifulSoup

from dataclasses import dataclass, field
from functools import cache, cached_property
from pathlib import Path


from .exceptions import (
    ConfigException,
    ConnectionException,
    FileException,
    ParserException,
)
from .type_hints import Optional, Union, PathLike
from .utils import clean_url, soupify, popkwargs, validate_path


@dataclass(unsafe_hash=True)
class ConfigFileParser:
    """
    A class to parse configuration files.
    
    #### Attributes:
        - `file_name`: The name of the file to parse.
        - `section`: The section to parse.
    
    #### Properties:
        - `full_path`: The full path of the configuration file.
        - `full_config`: The full configuration of the file.
    
    #### Raises:
        - `FileException`: If there is an error with the file.
        - `ParserException`: If there is an error with the parser.
    
    #### Notes:
        - The `file_name` attribute must be a valid file.
        - The `section` attribute must be a valid section in the configuration file.
    """
    
    file_name: PathLike = "config.ini"
    section: str = ""

    _config_file_path: Path = field(init=False, repr=False, default=None)
    _full_config: Union[dict, str] = field(init=False, repr=False, default=None)

    def __post_init__(self):
        for key, value in self.full_config.items():
            setattr(self, key, value)

    def __repr__(self):
        return f"{self.full_config}"

    def __iter__(self):
        return iter(self.full_config.items())

    def __getitem__(self, section):
        try:
            return self._full_config[section]
        except IndexError as idxe:
            raise idxe
        except KeyError:
            raise ConfigException(
                f"The provided section {section!r} is not a valid section in the config file."
            )

    @staticmethod
    def _check_args(fp: PathLike, section: str = ""):
        valid_path = validate_path(fp, is_file=True)

        if not isinstance(section, str):
            raise FileException(
                f"The provided section {section!r} is not a valid section and must be a string."
                "\n If a section is proivided, it must be a valid section in the config file."
            )
        else:
            valid_section = section.lower()
        return valid_path, valid_section

    @cache
    def _get_config(self, *args):
        fp, section = self._check_args(*args)

        self._config_file_path = fp

        try:
            config_p = tomllib.load(open(fp, "rb"))
        except Exception as e:
            raise ParserException(
                f"There was an error parsing the config file {fp!r}." f"\n{e}"
            )

        full_config = {k.lower(): dict(v) for k, v in dict(config_p).items()}

        if section:
            if section not in full_config:
                raise FileException(
                    f"The provided section {section!r} is not a valid section in the config file."
                )
            elif section in full_config:
                return full_config[section]

        return full_config

    @cached_property
    def full_path(self):
        return self._config_file_path

    @cached_property
    def full_config(self):
        if self._full_config is None:
            self._full_config = self._get_config(self.file_name, self.section)
        return self._full_config


@dataclass
class APIParser:
    """
    A class to parse API requests and responses.
    
    #### Attributes:
        - `url`: The URL to parse.
        - `endpoint`: The endpoint to parse.
        - `api_key`: The API key to use.
        - `json_format`: The format to parse the contents in.
        - `rapid_api`: A boolean indicating if the request is from RapidAPI.
        - `headers`: The headers to use for the request.
    
    #### Methods:
        - `main_request`: Make the main request to the URL.
        - `url_request`: Make the request to the URL.
        - `parse_html_contents`: Parse the HTML contents.
    
    #### Properties:
        - `contents`: The contents of the request.
    
    #### Raises:
        - `ConnectionException`: If there is an error with the connection.
        - `ParserException`: If there is an error with the parser.
    """
    
    url: str = None
    endpoint: Optional[str] = ""
    api_key: Optional[str] = field(default=None, repr=False, kw_only=True)
    json_format: Optional[str] = field(default=False, repr=False, kw_only=True)
    rapid_api: Optional[bool] = field(default=False, repr=False, kw_only=True)
    headers: Optional[dict[str, str]] = field(default_factory=dict, kw_only=True)

    _host: str = field(init=False, repr=False, default=None)
    _soupify = staticmethod(soupify)

    def __post_init__(self):
        self.url, self.api_key, self.headers, self._host = self._validate_args()
        self._contents = None

    @classmethod
    async def url_request(cls, url: str, endpoint: str = "", **kwargs):
        _json, headers, kwargs = popkwargs("json_format", "headers", **kwargs)

        if endpoint:
            url = "/".join((url.rstrip("/"), endpoint))

        try:
            async with ClientSession(
                connector=TCPConnector(
                    ssl=False,
                    enable_cleanup_closed=True,
                    force_close=True,
                    ttl_dns_cache=300,
                ),
                raise_for_status=True,
            ) as session:
                async with session.get(url, headers=headers) as response:
                    return await cls._url_contents(response, _json)
        except (ClientResponseError, ContentTypeError) as ccre:
            raise ccre
        except (ClientConnectionError, ServerDisconnectedError):
            return await cls.url_request(url, headers=headers, json_format=_json)
        except InvalidURL:
            raise ConnectionException(
                f"The specified URL could not be found and is considered invalid.",
                f"{url!r}",
                f"\n[Note]: Many queries may need the year (YYYY) to be included.",
                f"E.g 'The Matrix 1999'",
            )

    @staticmethod
    @alru_cache
    async def _url_contents(response, json_format=False):
        contents = await getattr(response, ["text", "json"][bool(json_format)])()
        return (
            contents[0] if all((json_format, isinstance(contents, list))) else contents
        )

    def main_request(self, **kwargs):
        try:
            url_contents = asyncio.run(
                self.url_request(self.url, self.endpoint, **kwargs)
            )
        except ClientResponseError as cre:
            raise ConnectionException(
                f"An error occured while trying to find {self.url!r}."
                f"\n[Original Error]: {cre}"
            )
        return url_contents

    @classmethod
    def parse_html_contents(cls, html_contents=None):
        if any((html_contents is None, isinstance(html_contents, BeautifulSoup))):
            raise ParserException(
                f"The provided contents cannot be {None!r} nor an instance of {BeautifulSoup!r}."
            )
        return cls._soupify(html_contents)

    @staticmethod
    @cache
    def rapidapi_headers(api_key="", host=""):
        return {"x-rapidapi-key": api_key, "x-rapidapi-host": host}

    def _validate_args(self):
        url_and_apikey = self.url, self.api_key
        headers = self.headers
        host = clean_url(url_and_apikey[0])

        if all((*url_and_apikey,)):
            if not any((*map(lambda x: isinstance(x, str), url_and_apikey),)):
                raise ParserException(
                    f"Both the endpoint and api-key must be strings and cannot be empty."
                )

        if self.rapid_api:
            self.json_format = True
            headers = self.rapidapi_headers(url_and_apikey[1], host)

        return *url_and_apikey, headers, host

    @cached_property
    def contents(self):
        if self._contents is None:
            self._contents = self.main_request(
                json_format=self.json_format, headers=self.headers
            )
        return self._contents


def _metadata_parser():
    pyproject_path = validate_path(Path(__file__).parents[3] / "pyproject.toml")
    return ConfigFileParser(pyproject_path).metadata


__all__ = ("APIParser", "ConfigFileParser")



from .trakt_hub import TraktHub, TraktHubViewer
from .trakt_utils.parsers import APIParser, ConfigFileParser, _metadata_parser
from .trakt_functions import (
    get_anticipated,
    get_popular,
    get_trending,
    is_anticipated,
    is_popular,
    is_trending,
    trakt_query,
)


# pyproject.toml
_pyproject = _metadata_parser()

# Package Metadata
__license__ = f"{_pyproject['license']}, Version 2.0"
__version__, __author__, __summary__, __url__ = (
    _pyproject.get(k) for k in ("version", "author", "description", "url")
)
__copyright__ = f"Copyright © 2024, {__author__}"


__all__ = (
    "APIParser",
    "ConfigFileParser",
    "TraktHub",
    "TraktHubViewer",
    "get_anticipated",
    "get_popular",
    "get_trending",
    "is_anticipated",
    "is_popular",
    "is_trending",
    "trakt_query",
    "__license__",
    "__version__",
    "__author__",
    "__summary__",
    "__url__",
    "__copyright__",
)
